package de.java2html.eclipse.preferences;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import de.java2html.eclipse.util.AwtToSwtColorTable;
import de.java2html.javasource.JavaSource;
import de.java2html.javasource.JavaSourceParser;
import de.java2html.javasource.JavaSourceType;
import de.java2html.options.JavaSourceConversionOptions;
import de.java2html.options.JavaSourceStyleEntry;
import de.java2html.options.JavaSourceStyleTable;

import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.TypedEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

/**
 * @author Markus Gebhard
 */
public class StylePreview implements MouseListener, SelectionListener {

  private static final String LINE_BREAK = "\r\n"; //$NON-NLS-1$

  public final static String DEMO_SOURCE = "/**"
      + LINE_BREAK
      + " * This is about <code>ClassName</code>."
      + LINE_BREAK
      + " * {@link com.yourCompany.aPackage.Interface}"
      + LINE_BREAK
      + " * @author author"
      + LINE_BREAK
      +
      //  " * @deprecated use <code>OtherClass</code>"+LINE_BREAK +
      " */"
      + LINE_BREAK
      + "public class ClassName<E> implements Interface<String> {"
      + LINE_BREAK
      + "  enum Color { RED, GREEN, BLUE };"
      + LINE_BREAK
      + "  /* This comment may span multiple lines. */"
      + LINE_BREAK
      + "  static Object staticField;"
      + LINE_BREAK
      + "  public final static char character = 'A';"
      + LINE_BREAK
      + "  // This comment may span only this line"
      + LINE_BREAK
      + "  private E field;"
      + LINE_BREAK
      + "  // TASK: refactor"
      + LINE_BREAK
      + "  @SuppressWarnings(value=\"all\")"
      + LINE_BREAK
      + "  public int foo(Integer parameter) {"
      + LINE_BREAK
      + "      abstractMethod();"
      + LINE_BREAK
      + "      int local= 42 * hashCode();"
//      + LINE_BREAK
//      + "      staticMethod();"
      + LINE_BREAK
      + "      return bar(local) + parameter;"
      + LINE_BREAK
      + "  }"
      + LINE_BREAK
      + "}";

  private JavaSource demoSource;
  private StyledText stCodePreview;
  private StyledText stLineNumberPreview;
  private final AwtToSwtColorTable colorTable;
  private List listeners;

  public StylePreview() {
    StringReader reader = new StringReader(DEMO_SOURCE);
    try {
      JavaSourceParser parser = new JavaSourceParser(JavaSourceConversionOptions.getDefault());
      demoSource = parser.parse(reader);
    }
    catch (Exception e) {
      System.err.println(e);
      e.printStackTrace();
    }
    colorTable = new AwtToSwtColorTable();
  }

  public synchronized void addSourceTypeSelectionListener(SourceTypeSelectionListener listener) {
    if (listeners == null) {
      listeners = new ArrayList();
    }
    listeners.add(listener);
  }

  public synchronized void removeSourceTypeSelectionListener(SourceTypeSelectionListener listener) {
    listeners.remove(listener);
  }

  public Control createControl(Composite parent) {
    Composite compo = new Composite(parent, SWT.NONE);
    GridLayout gridLayout = new GridLayout(2, false);
    gridLayout.marginHeight = 0;
    gridLayout.marginWidth = 0;
    gridLayout.horizontalSpacing = 0;
    compo.setLayout(gridLayout);

    stLineNumberPreview = new StyledText(compo, SWT.BORDER);
    stLineNumberPreview.setText(createLineNumbers(DEMO_SOURCE));
    stLineNumberPreview.setEditable(false);
    stLineNumberPreview.setFont(JFaceResources.getFontRegistry().get(JFaceResources.TEXT_FONT));
    stLineNumberPreview.addMouseListener(this);
    stLineNumberPreview.addSelectionListener(this);

    stCodePreview = new StyledText(compo, SWT.BORDER);
    stCodePreview.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    stCodePreview.setText(DEMO_SOURCE);
    stCodePreview.setEditable(false);
    stCodePreview.setFont(JFaceResources.getFontRegistry().get(JFaceResources.TEXT_FONT));
    stCodePreview.addMouseListener(this);
    stCodePreview.addSelectionListener(this);

    return compo;
  }

  public void setOptions(JavaSourceConversionOptions options) {
    if (stCodePreview.isDisposed() || stLineNumberPreview.isDisposed()) {
      return;
    }
    stCodePreview.setBackground(colorTable.getSwtColor(options.getStyleTable().get(
        JavaSourceType.BACKGROUND.getName()).getColor()));
    for (int i = 0; i < demoSource.getCode().length(); ++i) {
      JavaSourceType type = demoSource.getClassification()[i];
      JavaSourceStyleEntry style = options.getStyleTable().get(type.getName());
      Color foreground = colorTable.getSwtColor(style.getColor());
      int fontStyle = SWT.NORMAL;
      if (style.isItalic()) {
        fontStyle |= SWT.ITALIC;
      }
      if (style.isBold()) {
        fontStyle |= SWT.BOLD;
      }
      StyleRange range = new StyleRange(i, 1, foreground, null, fontStyle);
      stCodePreview.setStyleRange(range);
    }

    JavaSourceStyleEntry style = options.getStyleTable().get(JavaSourceType.BACKGROUND.getName());
    stLineNumberPreview.setBackground(colorTable.getSwtColor(style.getColor()));
    style = options.getStyleTable().get(JavaSourceType.LINE_NUMBERS.getName());
    stLineNumberPreview.setForeground(colorTable.getSwtColor(style.getColor()));
    int fontStyle = SWT.NORMAL;
    if (style.isItalic()) {
      fontStyle |= SWT.ITALIC;
    }
    if (style.isBold()) {
      fontStyle |= SWT.BOLD;
    }
    StyleRange range = new StyleRange(0, createLineNumbers(DEMO_SOURCE).length(), null, null, fontStyle);
    stLineNumberPreview.setStyleRange(range);
  }

  public void mouseDoubleClick(MouseEvent event) {
    //nothing to do
  }

  public void mouseDown(MouseEvent event) {
    previewCaretModified(event);
  }

  public void mouseUp(MouseEvent event) {
    //nothing to do
  }

  public void widgetSelected(SelectionEvent event) {
    previewCaretModified(event);
  }

  public void widgetDefaultSelected(SelectionEvent e) {
    //nothing to do
  }

  private void previewCaretModified(TypedEvent event) {
    SourceTypeSelectionEvent typeEvent = new SourceTypeSelectionEvent(this);
    if (event.getSource() == stLineNumberPreview) {
      typeEvent.setJavaSourceType(JavaSourceType.LINE_NUMBERS);
    }
    else if (event.getSource() == stCodePreview) {
      int caretOffset = stCodePreview.getCaretOffset();
      if (caretOffset >= DEMO_SOURCE.length() || DEMO_SOURCE.charAt(caretOffset) == 13) {
        typeEvent.setJavaSourceType(JavaSourceType.BACKGROUND);
      }
      else {
        typeEvent.setJavaSourceType(demoSource.getClassification()[caretOffset]);
      }
    }
    fire(typeEvent);
  }

  private synchronized void fire(SourceTypeSelectionEvent sourceTypeSelectionEvent) {
    for (int i = 0; i < listeners.size(); ++i) {
      ((SourceTypeSelectionListener) listeners.get(i)).selectionChanged(sourceTypeSelectionEvent);
    }
  }

  public static void main(String[] args) {
    Display display = new Display();
    Shell shell = new Shell(display);

    final StylePreview view = new StylePreview();
    Control control = view.createControl(shell);
    control.setBounds(shell.getClientArea());
    view.setOptions(JavaSourceConversionOptions.getDefault());

    shell.setSize(420, 190);
    shell.open();
    shell.setText("StylePreview");

    JavaSourceConversionOptions options1 = JavaSourceConversionOptions.getDefault();
    options1.setStyleTable(JavaSourceStyleTable.getDefaultKawaStyleTable());

    while (!shell.isDisposed()) {
      if (!display.readAndDispatch()) {
        display.sleep();
      }
    }
    display.dispose();
  }

  public static String createLineNumbers(String sourceCode) {
    int lineBreakCount = countLineBreaks(sourceCode);
    StringBuffer sb = new StringBuffer();
    for (int i = 1; i <= lineBreakCount + 1; ++i) {
      if (i > 1) {
        sb.append(LINE_BREAK);
      }
      sb.append(i < 10 ? "0" + i : String.valueOf(i)); //$NON-NLS-1$
    }
    return sb.toString();
  }

  private static int countLineBreaks(String str) {
    int count = 0;
    int index = 0;
    while ((index = str.indexOf(LINE_BREAK, index)) != -1) {
      count++;
      index += LINE_BREAK.length();
    }
    return count;
  }
  
  public void dispose() {
    colorTable.disposeColors();
  }
}